package cn.com.infosec.jce.provider;

import cn.com.infosec.asn1.pkcs.PKCSObjectIdentifiers;
import cn.com.infosec.crypto.AsymmetricBlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.Digest;
import cn.com.infosec.crypto.InvalidCipherTextException;
import cn.com.infosec.crypto.encodings.ISO9796d1Encoding;
import cn.com.infosec.crypto.encodings.OAEPEncoding;
import cn.com.infosec.crypto.encodings.PKCS1Encoding;
import cn.com.infosec.crypto.engines.RSABlindedEngine;
import cn.com.infosec.crypto.params.ParametersWithRandom;
import cn.com.infosec.crypto.params.RSAKeyParameters;
import cn.com.infosec.util.Strings;
import java.io.ByteArrayOutputStream;
import java.security.AlgorithmParameters;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.InvalidParameterException;
import java.security.Key;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.security.interfaces.RSAPrivateKey;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.AlgorithmParameterSpec;
import java.security.spec.MGF1ParameterSpec;
import javax.crypto.BadPaddingException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.spec.OAEPParameterSpec;
import javax.crypto.spec.PSource;
import javax.crypto.spec.PSource.PSpecified;

public class JCERSACipher extends WrapCipherSpi {
   // $FF: synthetic field
   static Class class$0;
   private AsymmetricBlockCipher cipher;
   private AlgorithmParameterSpec paramSpec;
   private AlgorithmParameters engineParams;
   private boolean publicKeyOnly;
   private boolean privateKeyOnly;
   private ByteArrayOutputStream bOut;

   public JCERSACipher(AsymmetricBlockCipher engine) {
      JCERSACipher var10000 = this;
      JCERSACipher var10001 = this;
      super();
      this.publicKeyOnly = false;
      this.privateKeyOnly = false;
      ByteArrayOutputStream var2;
      var2 = new ByteArrayOutputStream.<init>();
      var10001.bOut = var2;
      var10000.cipher = engine;
   }

   public JCERSACipher(OAEPParameterSpec pSpec) {
      JCERSACipher var10000 = this;
      OAEPParameterSpec var10001 = pSpec;
      super();
      this.publicKeyOnly = false;
      this.privateKeyOnly = false;
      this.bOut = new ByteArrayOutputStream();

      try {
         var10000.initFromSpec(var10001);
      } catch (NoSuchPaddingException var2) {
         throw new IllegalArgumentException(var2.getMessage());
      }
   }

   public JCERSACipher(boolean publicKeyOnly, boolean privateKeyOnly, AsymmetricBlockCipher engine) {
      JCERSACipher var10000 = this;
      JCERSACipher var10001 = this;
      JCERSACipher var10002 = this;
      JCERSACipher var10003 = this;
      super();
      this.publicKeyOnly = false;
      this.privateKeyOnly = false;
      ByteArrayOutputStream var4;
      var4 = new ByteArrayOutputStream.<init>();
      var10003.bOut = var4;
      var10002.publicKeyOnly = publicKeyOnly;
      var10001.privateKeyOnly = privateKeyOnly;
      var10000.cipher = engine;
   }

   private void initFromSpec(OAEPParameterSpec pSpec) {
      MGF1ParameterSpec var2;
      Digest var3;
      if ((var3 = JCEDigestUtil.getDigest((var2 = (MGF1ParameterSpec)pSpec.getMGFParameters()).getDigestAlgorithm())) != null) {
         this.cipher = new OAEPEncoding(new RSABlindedEngine(), var3, ((PSource.PSpecified)pSpec.getPSource()).getValue());
         this.paramSpec = pSpec;
      } else {
         throw new NoSuchPaddingException("no match on OAEP constructor for digest algorithm: " + var2.getDigestAlgorithm());
      }
   }

   protected int engineGetBlockSize() {
      try {
         return this.cipher.getInputBlockSize();
      } catch (NullPointerException var1) {
         throw new IllegalStateException("RSA Cipher not initialised");
      }
   }

   protected byte[] engineGetIV() {
      return null;
   }

   protected int engineGetKeySize(Key key) {
      if (key instanceof RSAPrivateKey) {
         return ((RSAPrivateKey)key).getModulus().bitLength();
      } else if (key instanceof RSAPublicKey) {
         return ((RSAPublicKey)key).getModulus().bitLength();
      } else {
         throw new IllegalArgumentException("not an RSA key!");
      }
   }

   protected int engineGetOutputSize(int inputLen) {
      try {
         return this.cipher.getOutputBlockSize();
      } catch (NullPointerException var2) {
         throw new IllegalStateException("RSA Cipher not initialised");
      }
   }

   protected AlgorithmParameters engineGetParameters() {
      if (this.engineParams == null && this.paramSpec != null) {
         Exception var10000;
         label31: {
            boolean var10001;
            AlgorithmParameters var4;
            try {
               var4 = AlgorithmParameters.getInstance("OAEP", "INFOSEC");
            } catch (Exception var3) {
               var10000 = var3;
               var10001 = false;
               break label31;
            }

            AlgorithmParameters var1 = var4;

            try {
               this.engineParams = var1;
               var4.init(this.paramSpec);
               return this.engineParams;
            } catch (Exception var2) {
               var10000 = var2;
               var10001 = false;
            }
         }

         Exception var5 = var10000;
         throw new RuntimeException(var5.toString());
      } else {
         return this.engineParams;
      }
   }

   protected void engineSetMode(String mode) {
      String var2;
      if (!(var2 = Strings.toUpperCase(mode)).equals("NONE") && !var2.equals("ECB")) {
         if (var2.equals("1")) {
            this.privateKeyOnly = true;
            this.publicKeyOnly = false;
         } else if (var2.equals("2")) {
            this.privateKeyOnly = false;
            this.publicKeyOnly = true;
         } else {
            throw new NoSuchAlgorithmException("can't support mode " + mode);
         }
      }
   }

   protected void engineSetPadding(String padding) {
      String var2;
      JCERSACipher var10000;
      if ((var2 = Strings.toUpperCase(padding)).equals("NOPADDING")) {
         var10000 = this;
         RSABlindedEngine var3;
         var3 = new RSABlindedEngine.<init>();
         var10000.cipher = var3;
      } else {
         RSABlindedEngine padding1;
         if (var2.equals("PKCS1PADDING")) {
            var10000 = this;
            PKCS1Encoding var4;
            PKCS1Encoding var10001 = var4 = new PKCS1Encoding;
            padding1 = new RSABlindedEngine.<init>();
            var10001.<init>(padding1);
            var10000.cipher = var4;
         } else if (var2.equals("ISO9796-1PADDING")) {
            var10000 = this;
            ISO9796d1Encoding var5;
            ISO9796d1Encoding var10 = var5 = new ISO9796d1Encoding;
            padding1 = new RSABlindedEngine.<init>();
            var10.<init>(padding1);
            var10000.cipher = var5;
         } else {
            OAEPParameterSpec var7;
            MGF1ParameterSpec padding2;
            PSource.PSpecified var9;
            OAEPParameterSpec var11;
            if (var2.equals("OAEPWITHMD5ANDMGF1PADDING")) {
               var10000 = this;
               var11 = var7 = new OAEPParameterSpec;
               padding2 = new MGF1ParameterSpec.<init>("MD5");
               var9 = PSpecified.DEFAULT;
               var11.<init>("MD5", "MGF1", padding2, var9);
               var10000.initFromSpec(var7);
            } else if (var2.equals("OAEPPADDING")) {
               this.initFromSpec(OAEPParameterSpec.DEFAULT);
            } else if (var2.equals("OAEPWITHSHA1ANDMGF1PADDING")) {
               this.initFromSpec(OAEPParameterSpec.DEFAULT);
            } else if (var2.equals("OAEPWITHSHA224ANDMGF1PADDING")) {
               var10000 = this;
               var11 = var7 = new OAEPParameterSpec;
               padding2 = new MGF1ParameterSpec.<init>("SHA-224");
               var9 = PSpecified.DEFAULT;
               var11.<init>("SHA-224", "MGF1", padding2, var9);
               var10000.initFromSpec(var7);
            } else if (var2.equals("OAEPWITHSHA256ANDMGF1PADDING")) {
               var10000 = this;
               var11 = var7 = new OAEPParameterSpec;
               padding2 = MGF1ParameterSpec.SHA256;
               var9 = PSpecified.DEFAULT;
               var11.<init>("SHA-256", "MGF1", padding2, var9);
               var10000.initFromSpec(var7);
            } else if (var2.equals("OAEPWITHSHA384ANDMGF1PADDING")) {
               var10000 = this;
               var11 = var7 = new OAEPParameterSpec;
               padding2 = MGF1ParameterSpec.SHA384;
               var9 = PSpecified.DEFAULT;
               var11.<init>("SHA-384", "MGF1", padding2, var9);
               var10000.initFromSpec(var7);
            } else {
               if (!var2.equals("OAEPWITHSHA512ANDMGF1PADDING")) {
                  throw new NoSuchPaddingException(padding + " unavailable with RSA.");
               }

               var10000 = this;
               var11 = var7 = new OAEPParameterSpec;
               padding2 = MGF1ParameterSpec.SHA512;
               var9 = PSpecified.DEFAULT;
               var11.<init>("SHA-512", "MGF1", padding2, var9);
               var10000.initFromSpec(var7);
            }
         }
      }

   }

   protected void engineInit(int opmode, Key key, AlgorithmParameterSpec params, SecureRandom random) {
      if (params != null && !(params instanceof OAEPParameterSpec)) {
         throw new IllegalArgumentException("unknown parameter type.");
      } else {
         RSAKeyParameters key1;
         if (key instanceof RSAPublicKey) {
            if (this.privateKeyOnly) {
               throw new InvalidKeyException("mode 1 requires RSAPrivateKey");
            }

            key1 = RSAUtil.generatePublicKeyParameter((RSAPublicKey)key);
         } else {
            if (!(key instanceof RSAPrivateKey)) {
               throw new InvalidKeyException("unknown key type passed to RSA");
            }

            if (this.publicKeyOnly) {
               throw new InvalidKeyException("mode 2 requires RSAPublicKey");
            }

            key1 = RSAUtil.generatePrivateKeyParameter((RSAPrivateKey)key);
         }

         if (params != null) {
            AlgorithmParameterSpec var10001 = params;
            OAEPParameterSpec params1 = (OAEPParameterSpec)params;
            this.paramSpec = var10001;
            if (!params1.getMGFAlgorithm().equalsIgnoreCase("MGF1") && !params1.getMGFAlgorithm().equals(PKCSObjectIdentifiers.id_mgf1.getId())) {
               throw new InvalidAlgorithmParameterException("unknown mask generation function specified");
            }

            if (!(params1.getMGFParameters() instanceof MGF1ParameterSpec)) {
               throw new InvalidAlgorithmParameterException("unkown MGF parameters");
            }

            MGF1ParameterSpec var5;
            if (!JCEDigestUtil.isSameDigest((var5 = (MGF1ParameterSpec)params1.getMGFParameters()).getDigestAlgorithm(), params1.getDigestAlgorithm())) {
               throw new InvalidAlgorithmParameterException("digest algorithm for MGF should be the same as for OAEP parameters.");
            }

            Digest var6;
            if ((var6 = JCEDigestUtil.getDigest(var5.getDigestAlgorithm())) == null) {
               throw new InvalidAlgorithmParameterException("no match on MGF digest algorithm: " + var5.getDigestAlgorithm());
            }

            this.cipher = new OAEPEncoding(new RSABlindedEngine(), var6, ((PSource.PSpecified)params1.getPSource()).getValue());
         }

         Object params2;
         if (!(this.cipher instanceof RSABlindedEngine)) {
            if (random != null) {
               params2 = new ParametersWithRandom.<init>(key1, random);
            } else {
               Object var10000 = params2 = new ParametersWithRandom;
               random = new SecureRandom.<init>();
               var10000.<init>(key1, random);
            }
         } else {
            params2 = key1;
         }

         switch (opmode) {
            case 1:
            case 3:
               this.cipher.init(true, (CipherParameters)params2);
               break;
            case 2:
            case 4:
               this.cipher.init(false, (CipherParameters)params2);
               break;
            default:
               throw new InvalidParameterException("unknown opmode " + opmode + " passed to RSA");
         }

      }
   }

   protected void engineInit(int param1, Key param2, AlgorithmParameters param3, SecureRandom param4) {
      // $FF: Couldn't be decompiled
   }

   protected void engineInit(int opmode, Key key, SecureRandom random) {
      try {
         this.engineInit(opmode, key, (AlgorithmParameterSpec)null, random);
      } catch (InvalidAlgorithmParameterException var4) {
         throw new RuntimeException("Eeeek! " + var4.toString(), var4);
      }
   }

   protected byte[] engineUpdate(byte[] input, int inputOffset, int inputLen) {
      this.bOut.write(input, inputOffset, inputLen);
      if (this.cipher instanceof RSABlindedEngine) {
         if (this.bOut.size() > this.cipher.getInputBlockSize() + 1) {
            throw new ArrayIndexOutOfBoundsException("too much data for RSA block");
         }
      } else if (this.bOut.size() > this.cipher.getInputBlockSize()) {
         throw new ArrayIndexOutOfBoundsException("too much data for RSA block");
      }

      return null;
   }

   protected int engineUpdate(byte[] input, int inputOffset, int inputLen, byte[] output, int outputOffset) {
      this.bOut.write(input, inputOffset, inputLen);
      if (this.cipher instanceof RSABlindedEngine) {
         if (this.bOut.size() > this.cipher.getInputBlockSize() + 1) {
            throw new ArrayIndexOutOfBoundsException("too much data for RSA block");
         }
      } else if (this.bOut.size() > this.cipher.getInputBlockSize()) {
         throw new ArrayIndexOutOfBoundsException("too much data for RSA block");
      }

      return 0;
   }

   protected byte[] engineDoFinal(byte[] input, int inputOffset, int inputLen) {
      if (input != null) {
         this.bOut.write(input, inputOffset, inputLen);
      }

      if (this.cipher instanceof RSABlindedEngine) {
         if (this.bOut.size() > this.cipher.getInputBlockSize() + 1) {
            throw new ArrayIndexOutOfBoundsException("too much data for RSA block");
         }
      } else if (this.bOut.size() > this.cipher.getInputBlockSize()) {
         throw new ArrayIndexOutOfBoundsException("too much data for RSA block");
      }

      InvalidCipherTextException var10000;
      label45: {
         JCERSACipher var10;
         JCERSACipher var12;
         boolean var10001;
         byte[] var10002;
         try {
            var10 = this;
            var12 = this;
            var10002 = this.bOut.toByteArray();
         } catch (InvalidCipherTextException var6) {
            var10000 = var6;
            var10001 = false;
            break label45;
         }

         byte[] var7 = var10002;

         AsymmetricBlockCipher var11;
         try {
            var12.bOut.reset();
            var11 = var10.cipher;
         } catch (InvalidCipherTextException var5) {
            var10000 = var5;
            var10001 = false;
            break label45;
         }

         byte[] var13 = var7;
         var10002 = var7;
         byte var8 = 0;

         try {
            return var11.processBlock(var13, var8, var10002.length);
         } catch (InvalidCipherTextException var4) {
            var10000 = var4;
            var10001 = false;
         }
      }

      InvalidCipherTextException var9 = var10000;
      throw new BadPaddingException(var9.getMessage());
   }

   protected int engineDoFinal(byte[] input, int inputOffset, int inputLen, byte[] output, int outputOffset) {
      if (input != null) {
         this.bOut.write(input, inputOffset, inputLen);
      }

      if (this.cipher instanceof RSABlindedEngine) {
         if (this.bOut.size() > this.cipher.getInputBlockSize() + 1) {
            throw new ArrayIndexOutOfBoundsException("too much data for RSA block");
         }
      } else if (this.bOut.size() > this.cipher.getInputBlockSize()) {
         throw new ArrayIndexOutOfBoundsException("too much data for RSA block");
      }

      byte[] var9;
      byte[] var17;
      label49: {
         InvalidCipherTextException var10000;
         label55: {
            JCERSACipher var13;
            JCERSACipher var15;
            boolean var10001;
            byte[] var10002;
            try {
               var13 = this;
               var15 = this;
               var10002 = this.bOut.toByteArray();
            } catch (InvalidCipherTextException var8) {
               var10000 = var8;
               var10001 = false;
               break label55;
            }

            var9 = var10002;

            AsymmetricBlockCipher var14;
            try {
               var15.bOut.reset();
               var14 = var13.cipher;
            } catch (InvalidCipherTextException var7) {
               var10000 = var7;
               var10001 = false;
               break label55;
            }

            byte[] var16 = var9;
            var10002 = var9;
            byte var10 = 0;

            try {
               var17 = var14.processBlock(var16, var10, var10002.length);
               break label49;
            } catch (InvalidCipherTextException var6) {
               var10000 = var6;
               var10001 = false;
            }
         }

         InvalidCipherTextException var11 = var10000;
         throw new BadPaddingException(var11.getMessage());
      }

      var9 = var17;

      for(int input1 = 0; input1 != var9.length; ++input1) {
         inputOffset = outputOffset + input1;
         output[inputOffset] = var9[input1];
      }

      return var9.length;
   }

   public static class ISO9796d1Padding extends JCERSACipher {
      public ISO9796d1Padding() {
         ISO9796d1Padding var10000 = this;
         ISO9796d1Encoding var2;
         ISO9796d1Encoding var10001 = var2 = new ISO9796d1Encoding;
         RSABlindedEngine var1;
         var1 = new RSABlindedEngine.<init>();
         var10001.<init>(var1);
         var10000.<init>((AsymmetricBlockCipher)var2);
      }
   }

   public static class NoPadding extends JCERSACipher {
      public NoPadding() {
         NoPadding var10000 = this;
         RSABlindedEngine var1;
         var1 = new RSABlindedEngine.<init>();
         var10000.<init>((AsymmetricBlockCipher)var1);
      }
   }

   public static class OAEPPadding extends JCERSACipher {
      public OAEPPadding() {
         super(OAEPParameterSpec.DEFAULT);
      }
   }

   public static class PKCS1v1_5Padding extends JCERSACipher {
      public PKCS1v1_5Padding() {
         PKCS1v1_5Padding var10000 = this;
         PKCS1Encoding var2;
         PKCS1Encoding var10001 = var2 = new PKCS1Encoding;
         RSABlindedEngine var1;
         var1 = new RSABlindedEngine.<init>();
         var10001.<init>(var1);
         var10000.<init>((AsymmetricBlockCipher)var2);
      }
   }

   public static class PKCS1v1_5Padding_PrivateOnly extends JCERSACipher {
      public PKCS1v1_5Padding_PrivateOnly() {
         PKCS1v1_5Padding_PrivateOnly var10000 = this;
         PKCS1Encoding var2;
         PKCS1Encoding var10001 = var2 = new PKCS1Encoding;
         RSABlindedEngine var1;
         var1 = new RSABlindedEngine.<init>();
         var10001.<init>(var1);
         var10000.<init>(false, true, var2);
      }
   }

   public static class PKCS1v1_5Padding_PublicOnly extends JCERSACipher {
      public PKCS1v1_5Padding_PublicOnly() {
         PKCS1v1_5Padding_PublicOnly var10000 = this;
         PKCS1Encoding var2;
         PKCS1Encoding var10001 = var2 = new PKCS1Encoding;
         RSABlindedEngine var1;
         var1 = new RSABlindedEngine.<init>();
         var10001.<init>(var1);
         var10000.<init>(true, false, var2);
      }
   }
}
